function [img, img2] = disegna_newton(m, n)
%DISEGNA_NEWTON Versione vettorizzata di NEWTONFRACTAL. 
%
% IMG = DISEGNA_NEWTON(M, N) crea una matrice M x N contenente il frattale 
%     di Newton per il polinomio x^3 + 1. Si può visualizzare l'immagine
%     tramite il comando IMAGESC(IMG). 
%
% [IMG, IMG2] = DISEGNA_NEWTON(M, N) ritorna anche una versione più
%     dettagliata dell'immagine, dove viene anche utilizzata l'intensità di
%     colore di ogni pixel per segnalare la velocità di convergenza. Questa
%     va visualizzata con il comando IMSHOW(IMG2);

% Scegliamo una scala appropriata in modo che [-1, 1]^2 sia contenuto
% nell'immagine, ed in modo che la scala sia uguale sui due assi. 
if n > m
    xscl = n / m;
    yscl = 1;
else
    xscl = 1;
    yscl = m / n;
end

x = linspace(-2 * xscl, 2 * xscl, n);
y = linspace(-2 * yscl, 2 * yscl, m);

maxits = 15;

img = ones(m, 1) * x + 1i * y' * ones(1, n);

p  = [ 1 0 0 1 ];
pd = polyder(p);

d = length(p) - 1;

% Manteniamo una matrix N x N dove teniamo traccia del numero di iterazioni
% necessarie per arrivare a convergenza. 
nits = zeros(m, n);

for it = 1 : maxits
    pv = polyval(p, img);
    img = img - pv ./ polyval(pd, img);    
    
    % Contiamo l'iterazione solo se p(x) non è già piccolo. 
    nits = nits + ( pv > 1e-12 );
end

% Determiniamo a quale approssimazione siamo andati più vicini - per fare
% questo ci appoggiamo ad un tensore N x N x D, dove D è il grado del
% polinomio. 
xi = roots(p);

dist = zeros(m, n, d);

for i = 1 : d
    dist(:, :, i) = abs(img - xi(i));
end

% Prendiamo il minimo sulla terza componente (help min per i dettagli). 
[~, img] = min(dist, [], 3);

% Giochiamo un po' con i colori: creiamo un'immagine in cui ogni pixel ha
% un colore diverso (scelto a caso) determinato dalla radice a cui c'è
% convergenza. In particolare, l'intensità luminosa del pixel è data dalla
% velocità di convergenza. 
img2 = zeros(m, n, 3);
colors = rand(d, 3);

for i = 1 : d
    % Selezioniamo il colore scelto per la radice i
    c = colors(i, :);
    
    % Identifichiamo le componenti che sono arrivata a convergenza in
    % quella radice (come sono disposti questi indici?)
    ixy = find(img == i);
    
    for j = 1 : 3
        % Un po' di black-magic nell'indexing... cosa fa quest'istruzione?
        img2(ixy + (j-1)*m*n) = c(j) .* (maxits - nits(ixy)) / maxits;
    end
end

end

